/*
 * PRZYCISKI.c
 *
 *  Created on: 14.01.2017
 *      Author: Przemek
 */


#include <PRZYCISKI/przyciski.h>
#include "stm32f0xx.h"

// flaga od pobrania nowego stanu przyciskow
uint8_t buttons_flag = 0;

typedef struct
{
	uint8_t obecny_stan;

	uint8_t debounce_time_high;
	uint16_t repeat_time_first;
	uint16_t repeat_time_next;

	uint16_t h_counter;
	uint16_t h_counter2;

	// callbacki do funkcji
	button_Callback_type * push_function;
	button_Callback_type * repeat_function;
}BUTTON;

volatile BUTTON tablica_przyciskow[ BUTTONS_NUMBER ];

void button_ustaw( uint8_t numer, uint8_t debounce_time_high, uint16_t repeat_time_first, uint16_t repeat_time_next, button_Callback_type* push_function, button_Callback_type* repeat_function )
{
	tablica_przyciskow[numer].debounce_time_high 	= debounce_time_high;
	tablica_przyciskow[numer].repeat_time_first 	= repeat_time_first;
	tablica_przyciskow[numer].repeat_time_next 		= repeat_time_next;
	tablica_przyciskow[numer].push_function 		= push_function;
	tablica_przyciskow[numer].repeat_function 		= repeat_function;
}

void init_buttons()
{
	BUTTON1_INIT;
	BUTTON1_PULL_UP;

	BUTTON2_INIT;
	BUTTON2_PULL_UP;

	BUTTON3_INIT;
	BUTTON3_PULL_UP;

	BUTTON4_INIT;
	BUTTON4_PULL_UP;

	BUTTON5_INIT;
	BUTTON5_PULL_UP;

	init_timer_buttons();
}

// inicjalizujemy nasz timer do obslugi przyciskow
// domyslnie ustawmy go co 5ms
void init_timer_buttons( void )
{
	// uruchamiamy zegar dla naszego timera
	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;

	// wpisujemy wartosc preskalera naszego timera
	// jest to rejest 16bitowy wpiszmy wiec 48000
	TIM_PSC(BUTTON_TIM) = 48000UL;

	// wpisujemy do jakiej wartosci ma zliczac timer
	TIM_ARR(BUTTON_TIM) = 5;

	// uruchamiamy przerwanie od timera
	TIM_DIER(BUTTON_TIM) |= TIM_DIER_UIE;

	// na wszelki wypadek do licznika mozemy wpisac 0
	TIM_CNT(BUTTON_TIM) = 0;

	// KONIEC KONFIGURACJI TIMERA

	// teraz konfigurujemy przerwanie
	NVIC_SetPriority( TIM3_IRQn, 5 );
	NVIC_EnableIRQ( TIM3_IRQn );

	// uruchamiamy timer
	TIM_CR1(BUTTON_TIM) |= TIM_CR1_CEN;
}

void buttons_check_events()
{
	BUTTON * actual_button;
	actual_button = &tablica_przyciskow[0];


	// jesli nastapil nowy odczyt:
	if( buttons_flag )
	{
		buttons_flag = 0;

		for( uint8_t i = 0; i < BUTTONS_NUMBER; i++ )
		{

			actual_button = &tablica_przyciskow[i];

			// jesli przycisk jest wduszony:
			if( actual_button->obecny_stan == 1 )
			{
				actual_button->h_counter++;

				// sprawdzenie czasu po wcisnieciu
				if( actual_button->h_counter == actual_button->debounce_time_high )
				{
					if( actual_button->push_function )
					{
						actual_button->push_function();
					}
				}

				if( actual_button->h_counter > actual_button->repeat_time_first )
				{
					actual_button->h_counter2++;

					if( actual_button->h_counter2 == actual_button->repeat_time_next )
					{
						if( actual_button->repeat_function )
						{
							actual_button->repeat_function();
						}
					}
				}
			}
			else
			{
				// wyzerowania licznika stanu wcisniecia
				actual_button->h_counter = 0;
				actual_button->h_counter2 = 0;
			}
		}
	}
}

// funkcja sprawdzajaca stany przyciskow
void buttons_check()
{
	buttons_flag = 1;

	if( BUTTON1_IS_LOW )
	{
		tablica_przyciskow[0].obecny_stan = 1;
	}
	else
	{
		tablica_przyciskow[0].obecny_stan = 0;
	}

	if( BUTTON2_IS_LOW )
	{
		tablica_przyciskow[1].obecny_stan = 1;
	}
	else
	{
		tablica_przyciskow[1].obecny_stan = 0;
	}

	if( BUTTON3_IS_LOW )
	{
		tablica_przyciskow[2].obecny_stan = 1;
	}
	else
	{
		tablica_przyciskow[2].obecny_stan = 0;
	}

	if( BUTTON4_IS_LOW )
	{
		tablica_przyciskow[3].obecny_stan = 1;
	}
	else
	{
		tablica_przyciskow[3].obecny_stan = 0;
	}

	if( BUTTON5_IS_LOW )
	{
		tablica_przyciskow[4].obecny_stan = 1;
	}
	else
	{
		tablica_przyciskow[4].obecny_stan = 0;
	}
}

// funkcja obslugi przerwania:
void BUTTON_TIM_INTERRUPT(void)
{
	// sprawdzamy, czy jest to nasze przerwanie od zliczenia
	if( TIM_SR(BUTTON_TIM) & TIM_SR_UIF)
	{
		// kasujemy flage przerwania, ale w tym przypadku przez wpisanie 0 :)
		TIM_SR(BUTTON_TIM) = ~TIM_SR_UIF;

		buttons_check();
	}
}
