/*
 * lcd_ILI9341.h
 *
 *  Created on: 2 sie 2014
 *      Author: Przemek
 */

#ifndef LCD_ILI9341_H_
#define LCD_ILI9341_H_

#include "stm32f0xx.h"
#include "DMB/rejestry.h"

#define LCD_DC_GPIO 	GPIOB
#define LCD_DC_PIN 		12

#define LCD_CE_GPIO 	GPIOB
#define LCD_CE_PIN 		10

#define LCD_RST_GPIO	GPIOB
#define LCD_RST_PIN 	11

#define LCD_LED_GPIO 	GPIOB
#define LCD_LED_PIN 	2

#define LCD_DC_OUT 		GPIO_MODER(LCD_DC_GPIO) |= ( 1 << ( 2 * LCD_DC_PIN));
#define LCD_DC_SET 		GPIO_BSRR(LCD_DC_GPIO) = (1 << LCD_DC_PIN);
#define LCD_DC_CLR		GPIO_BRR(LCD_DC_GPIO) = ( 1 << LCD_DC_PIN );

#define LCD_CE_OUT 		GPIO_MODER(LCD_CE_GPIO) |= ( 1 << ( 2 * LCD_CE_PIN));
#define LCD_CE_SET 		GPIO_BSRR(LCD_CE_GPIO) = (1 << LCD_CE_PIN);
#define LCD_CE_CLR 		GPIO_BRR(LCD_CE_GPIO) = ( 1 << LCD_CE_PIN );
#define LCD_CE_TOG 		GPIO_ODR(LCD_CE_GPIO) ^= ( 1 << LCD_CE_PIN );

#define LCD_RST_OUT 	GPIO_MODER(LCD_RST_GPIO) |= ( 1 << ( 2 * LCD_RST_PIN));
#define LCD_RST_SET 	GPIO_BSRR(LCD_RST_GPIO) = (1 << LCD_RST_PIN);
#define LCD_RST_CLR 	GPIO_BRR(LCD_RST_GPIO) = ( 1 << LCD_RST_PIN );
#define LCD_RST_TOG 	GPIO_ODR(LCD_RST_GPIO) ^= ( 1 << LCD_RST_PIN );

#define LCD_LED_OUT 	GPIO_MODER(LCD_LED_GPIO) |= ( 1 << ( 2 * LCD_LED_PIN));
#define LCD_LED_OFF 		GPIO_BSRR(LCD_LED_GPIO) = (1 << LCD_LED_PIN);
#define LCD_LED_ON 	GPIO_BRR(LCD_LED_GPIO) = ( 1 << LCD_LED_PIN );
#define LCD_LED_TOG 	GPIO_ODR(LCD_LED_GPIO) ^= ( 1 << LCD_LED_PIN );


#define lcd_komenda 0
#define lcd_dane 1



#define ILI9341_TFTWIDTH  240
#define ILI9341_TFTHEIGHT 320

#define ILI9341_NOP     0x00
#define ILI9341_SWRESET 0x01
#define ILI9341_RDDID   0x04
#define ILI9341_RDDST   0x09

#define ILI9341_SLPIN   0x10
#define ILI9341_SLPOUT  0x11
#define ILI9341_PTLON   0x12
#define ILI9341_NORON   0x13

#define ILI9341_RDMODE  0x0A
#define ILI9341_RDMADCTL  0x0B
#define ILI9341_RDPIXFMT  0x0C
#define ILI9341_RDIMGFMT  0x0A
#define ILI9341_RDSELFDIAG  0x0F

#define ILI9341_INVOFF  0x20
#define ILI9341_INVON   0x21
#define ILI9341_GAMMASET 0x26
#define ILI9341_DISPOFF 0x28
#define ILI9341_DISPON  0x29

#define ILI9341_CASET   0x2A
#define ILI9341_PASET   0x2B
#define ILI9341_RAMWR   0x2C
#define ILI9341_RAMRD   0x2E

#define ILI9341_PTLAR   0x30
#define ILI9341_MADCTL  0x36
#define ILI9341_PIXFMT  0x3A

#define ILI9341_FRMCTR1 0xB1
#define ILI9341_FRMCTR2 0xB2
#define ILI9341_FRMCTR3 0xB3
#define ILI9341_INVCTR  0xB4
#define ILI9341_DFUNCTR 0xB6

#define ILI9341_PWCTR1  0xC0
#define ILI9341_PWCTR2  0xC1
#define ILI9341_PWCTR3  0xC2
#define ILI9341_PWCTR4  0xC3
#define ILI9341_PWCTR5  0xC4
#define ILI9341_VMCTR1  0xC5
#define ILI9341_VMCTR2  0xC7

#define ILI9341_RDID1   0xDA
#define ILI9341_RDID2   0xDB
#define ILI9341_RDID3   0xDC
#define ILI9341_RDID4   0xDD

#define ILI9341_GMCTRP1 0xE0
#define ILI9341_GMCTRN1 0xE1

#define ILI9341_PWCTR6  0xFC

#define	ILI9341_BLACK   0x0000
#define	ILI9341_BLUE    0x001F
#define	ILI9341_RED     0xF800
#define	ILI9341_GREEN   0x07E0
#define ILI9341_CYAN    0x07FF
#define ILI9341_MAGENTA 0xF81F
#define ILI9341_YELLOW  0xFFE0
#define ILI9341_WHITE   0xFFFF



void lcd_wyslij_bajt(unsigned char d_c, unsigned char data );
void lcd_init(void);
void lcd_ustaw_obszar(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
void lcd_rysuj_piksel(uint16_t x, uint16_t y, uint16_t kolor);
void lcd_zapelnij_prostokat(uint16_t x, uint16_t y, uint16_t x1, uint16_t y1, uint16_t kolor);
void lcd_zapal_piksel(uint16_t kolor);
void lcd_pisz_znak(char znak, uint16_t kolor, uint16_t tlo);
void lcd_pisz_tekst_16(uint16_t x, uint16_t y, char * str,uint16_t kolor,uint16_t tlo);
void lcd_pisz_tekst_32(uint16_t x, uint16_t y, char * str,uint16_t kolor,uint16_t tlo);
void lcd_czysc_ekran(uint16_t kolor);
void lcd_pixel(uint16_t kolor);

#endif /* LCD_ILI9341_H_ */
